import { FederationServiceTypes } from '@cross-lab-project/api-client';
import { LitElement, html } from 'lit';
import { customElement, property, query } from 'lit/decorators.js';
import { apiClient } from '../../../globals';
import { Editor } from '../common';

@customElement('apitool-institution-editor')
export class InstitutionEditor extends LitElement {
    @property({ type: Object })
    institution!: FederationServiceTypes.Institution<'response'>;

    @query('apitool-editor')
    editor!: Editor;

    changes: FederationServiceTypes.Institution<'request'> = {};

    protected createRenderRoot(): Element | ShadowRoot {
        return this;
    }

    protected render(): unknown {
        return html`<apitool-editor
            .type=${'editor'}
            @update-resource=${this.updateInstitution}
            @delete-resource=${this.deleteInstitution}
            @cancel=${this.cancel}
        >
            <div class="flex">
                <p class="w-24 flex-shrink-0">URL:</p>
                <p class="whitespace-nowrap overflow-hidden text-ellipsis">
                    ${this.institution.url}
                </p>
            </div>
            <div class="flex flex-col">
                <p>Name:</p>
                <input
                    id="input-name"
                    type="text"
                    value=${this.institution.name}
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.name = this.changes.name = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>Homepage:</p>
                <input
                    id="input-homepage"
                    type="text"
                    value=${this.institution.homepage}
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.homepage = this.changes.homepage = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>API URL:</p>
                <input
                    id="input-api"
                    type="text"
                    value=${this.institution.api}
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.institution.api = this.changes.api = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
            <div class="flex flex-col">
                <p>API Token:</p>
                <input
                    id="input-api-token"
                    type="text"
                    class="w-full p-2 resize-none border rounded-lg"
                    @input=${(event: InputEvent) => {
                        this.changes.apiToken = (
                            event.target as HTMLInputElement
                        ).value;
                        this.editor.messageField.removeAllSuccessMessages();
                    }}
                />
            </div>
        </div>
        </apitool-editor>`;
    }

    private async updateInstitution() {
        console.log(
            'trying to update institution:',
            this.institution,
            this.changes
        );
        try {
            const updatedInstitution = await apiClient.updateInstitution(
                this.institution.url,
                this.changes
            );

            this.editor.messageField.addMessage(
                'success',
                'Institution updated successfully!'
            );

            this.institution = updatedInstitution;
            console.log(
                'instutution updated successfully:',
                updatedInstitution
            );
        } catch (error) {
            this.editor.messageField.addMessage(
                'error',
                JSON.stringify(error, null, 4),
                'update-error'
            );
        }
    }

    private async deleteInstitution() {
        console.log('trying to delete institution:', this.institution.url);
        try {
            await apiClient.deleteInstitution(this.institution.url);

            const event = new CustomEvent<string>('update-view', {
                detail: '/institutions',
                bubbles: true,
            });

            this.dispatchEvent(event);

            this.editor.messageField.addMessage(
                'success',
                'Institution deleted successfully!'
            );
            console.log('institution deleted successfully');
        } catch (error) {
            this.editor.messageField.addMessage(
                'error',
                JSON.stringify(error, null, 4),
                'delete-error'
            );
        }
    }

    private cancel() {
        const event = new CustomEvent<string>('update-view', {
            detail: '/institutions',
            bubbles: true,
        });

        this.dispatchEvent(event);
    }
}
