import { FederationServiceTypes } from '@cross-lab-project/api-client';
import { LitElement, html } from 'lit';
import { customElement, state } from 'lit/decorators.js';
import { renderLoadingScreen } from '../helper';
import { apiClient } from '../../../globals';
import { InstitutionFilterOptions } from './institution-list-view-filter';

@customElement('apitool-institution-list-view')
export class InstitutionListView extends LitElement {
    @state()
    isReady: boolean = false;

    @state()
    institutions: FederationServiceTypes.Institution<'response'>[] = [];

    @state()
    filteredInstitutions: FederationServiceTypes.Institution<'response'>[] = [];

    constructor() {
        super();

        this.initialize()
            .catch((error) => console.error(error))
            .finally(() => {
                this.isReady = true;
            });
    }

    protected createRenderRoot(): Element | ShadowRoot {
        return this;
    }

    private async initialize() {
        this.institutions = await apiClient.listInstitutions();
        this.filteredInstitutions = this.institutions;
    }

    protected render(): unknown {
        return html`${renderLoadingScreen(this.isReady)}
            <div
                class="w-full flex flex-col items-center ${!this.isReady
                    ? 'hidden'
                    : ''}"
            >
                <div
                    class="p-4 w-[60rem] max-w-full relative flex flex-col gap-2 flex-grow"
                >
                    <apitool-institution-list-view-filter
                        @filters-updated=${(
                            event: CustomEvent<InstitutionFilterOptions>
                        ) => this.filter(event.detail)}
                    ></apitool-institution-list-view-filter>
                    ${this.filteredInstitutions.map(
                        (institution) =>
                            html`<apitool-institution-list-view-item
                                @delete-institution=${this.deleteInstitution}
                                .institution=${institution}
                            ></apitool-institution-list-view-item>`
                    )}
                </div>
                <div
                    class="flex w-full justify-center p-2 sticky bottom-0 left-0 bg-white border-t border-black pb-[calc(0.5rem_+_env(safe-area-inset-bottom))]"
                >
                    <button
                        class="bg-green-300 p-2 rounded-full"
                        @click=${() => this.createInstitution()}
                    >
                        + Add Institution
                    </button>
                </div>
            </div>`;
    }

    private filter(filterOptions: InstitutionFilterOptions) {
        this.filteredInstitutions = this.institutions.filter((institution) => {
            if (
                institution.url.includes(filterOptions.url) &&
                institution.name
                    ?.toLowerCase()
                    .includes(filterOptions.name.toLowerCase()) &&
                institution.homepage?.includes(filterOptions.homepage) &&
                institution.api?.includes(filterOptions.api)
            )
                return true;
            else return false;
        });
    }

    private createInstitution() {
        const event = new CustomEvent<string>('update-view', {
            detail: '/institution_creation',
            bubbles: true,
        });

        this.dispatchEvent(event);
    }

    private async deleteInstitution(event: CustomEvent<string>) {
        await apiClient.deleteInstitution(event.detail);
        this.institutions = this.institutions.filter(
            (institution) => institution.url !== event.detail
        );
        this.filteredInstitutions = this.filteredInstitutions.filter(
            (institution) => institution.url !== event.detail
        );
    }
}
